/*=================================================================
 *
 * cond_loss_dist_sector_c.c ... Compute the conditional portfolio loss distribution (conditional on common risk factors Z) of a
 *                               portfolio of credits using the recursive method described in Eckner and Hossein (2007).
 *
 * The calling syntax is:
 *      cond_loss_dist_sector_c(ai, p_surv_idio, int_common_factor, sector_ind, bi, X_good, X_bad)
 *
 * ai                   ... row vector of loadings on common factor
 * p_surv_idio          ... row vector of idiosyncratic survival probabilities
 * int_common_factor    ... integrated common factor
 * sector_ind           ... row vector of (sorted) sector designations, i.e. 1,1,1,2,2,..
 * bi                   ... row vector of loadings on sector factor
 * X_good               ... row vector (of length S) with value of integrated sector factor in good sector state
 * X_bad                ... row vector (of length S) with value of integrated sector factor in bad sector state
 *
 * Sample call:
 *      cond_loss_dist_sector_c(ones(1,10), repmat(0.9, 1, 10), 0, [1 1 1 1 1 2 2 2 2 2], ones(1,10), [0 0], [0.1, 0.1])
 *      cond_loss_dist_sector_c(ones(1,10), repmat(0.95, 1, 10), 0.05, [1 1 1 1 1 2 2 2 2 2], ones(1,10), [0 0], [0.1, 0.1])
 *      cond_loss_dist_sector_c(ones(1,100), repmat(0.9, 1, 100), 0, [ones(1,50) ones(1,50)*2], ones(1,100), [0 0], [1, 5])
 *=================================================================*/

#include <math.h>
#include "mex.h"

#ifndef max
#define max(a,b) (((a) > (b)) ? (a) : (b))
#endif

/* Recursive computation of loss distribution */
static void cond_loss_c2(double p[], double loss[], unsigned int num_firms, unsigned int previous_firms)
{
    int i, j;

    for (i=0; i<num_firms; i++) 
    {        
        // Update loss distribution when adding one asset
        for (j=i+previous_firms+1; j>=1; j--)
        {
            loss[j] = loss[j]*(1-p[i]) + loss[j-1]*p[i];
        }
        loss[0] = loss[0] * (1-p[i]);
    }
    //loss[0] = previous_firms;
    return;
}


// Wrapper function around cond_loss_dist2, which is the ASB algorithm
static void cond_loss_dist_wrapper(double loss[], double ai[], double p_surv_idio[], double Y_int[], unsigned int num_firms,
                                   unsigned int num_sectors, double sector_ind[], double bi[], double X_good[], double X_bad[])
{
    int i, j, index, num_firms_sector, sector_start, sector_end = 0;
    double *loss_good, *loss_bad, *p_def_good, *p_def_bad;
    
    // Initialize loss distribution
    loss[0] = 1;
    for (i=1; i<num_firms+1; i++)
    {
        loss[i] = 0;
    }
    
    // Allocate memory for temporary variables
    p_def_good = mxGetPr(mxCreateDoubleMatrix(num_firms, 1, mxREAL));
    p_def_bad = mxGetPr(mxCreateDoubleMatrix(num_firms, 1, mxREAL));
    
    // Calculate conditional default probabilities in good and bad sector states
    for (i=0; i<num_firms; i++)
    {
        index = (int) sector_ind[i]-1;
        p_def_good[i] = 1 - p_surv_idio[i] * exp( -Y_int[0]*ai[i] - X_good[index]*bi[i]);
        p_def_bad[i] = 1 - p_surv_idio[i] * exp( -Y_int[0]*ai[i] - X_bad[index]*bi[i]);
    }
        
    // For each sector, do two separate modified ASB steps, and merge the results afterwards
    loss_good = mxGetPr(mxCreateDoubleMatrix(num_firms+1, 1, mxREAL));
    loss_bad = mxGetPr(mxCreateDoubleMatrix(num_firms+1, 1, mxREAL)); 
    for (i=0; i<num_sectors; i++)
    {
        // Copy current loss distribution in variables for good and bad sector state calculation
        for (j=0; j<num_firms+1; j++) 
        {
            loss_good[j] = loss[j];
            loss_bad[j] = loss[j];
        }        
        
        // Determine number of firms in j-th sector
        num_firms_sector = 0;
        sector_start = sector_end;
        while ((sector_ind[sector_end] <= i+1) && (sector_end < num_firms))
        {
            sector_end++;
            num_firms_sector++;
        }
        
        // Do modified ASB step for good and bad sector state
        cond_loss_c2(&p_def_good[sector_start], loss_good, num_firms_sector, sector_start);
        cond_loss_c2(&p_def_bad[sector_start], loss_bad, num_firms_sector, sector_start);
        
        // Merge results of good and bad sector state (using equal probability for now)
        for (j=0; j<num_firms+1; j++) 
        {
            loss[j] = (loss_good[j] + loss_bad[j]) / 2.0;
        }
    }
    
//     for (i=0; i<num_firms; i++)
//     {
//         loss[i] = loss_bad[i];
//     }
    
    // Free memory of temporary variables
    free(loss_good);
    free(loss_bad);
    
    return;
}

/* Gateway routine (to Matlab) */
/* sample call: horizon_loss_dist_c2([1; 2], [0.9 0.8], [0.9 0.05 0.05], 1) */
void mexFunction( int nlhs, mxArray *plhs[], 
		  int nrhs, const mxArray*prhs[] )
     
{ 
    double *ai, *p_surv_idio, *Y_int, *bi, *X_good, *X_bad, *sector_ind; 
    double *loss;
    unsigned int num_firms, num_sectors; 

    /* Get number of companies, number of points in time, and number of points for numerical integration */
    num_firms = mxGetN(prhs[1]);
    
    /* Create a matrix for the return argument */ 
    plhs[0] = mxCreateDoubleMatrix(num_firms+1, 1, mxREAL); 
    
    /* Assign pointers to the various parameters */ 
    loss = mxGetPr(plhs[0]);
    ai = mxGetPr(prhs[0]);
    p_surv_idio = mxGetPr(prhs[1]);
    Y_int = mxGetPr(prhs[2]);
    sector_ind = mxGetPr(prhs[3]);
    bi = mxGetPr(prhs[4]);
    X_good = mxGetPr(prhs[5]);
    X_bad = mxGetPr(prhs[6]);
    
    /* Determine number of sectors */
    num_sectors = mxGetN(prhs[5]);
    
    /* Do the actual computations in a subroutine */
    cond_loss_dist_wrapper(loss, ai, p_surv_idio, Y_int, num_firms, num_sectors, sector_ind, bi, X_good, X_bad);
    return;
}